/*************************************************************************
 OLIMEX MOD BT Client
 **************************************************************************/
#include "includes.h"
#include <yfuns.h>

void Delay_time (unsigned long time);

//bluetooth definitions
#define BT_ANSWER         5000000
#define DOT_TIME          3000000

#define DTE_RTS_DIR       (1<<8)
#define DTE_RTS_HIGH      GPIO0DATA |= (1<<8)     
#define DTE_RTS_LOW       GPIO0DATA &= ~(1<<8)     
#define DTE_DTR_DIR       (1<<10)
#define DTE_DTR_HIGH      GPIO2DATA |= (1<<10)     
#define DTE_DTR_LOW       GPIO2DATA &= ~(1<<10)
#define RST_DIR           (1<<5)
#define RST_HIGH          GPIO0DATA |= (1<<5)     
#define RST_LOW           GPIO0DATA &= ~(1<<5)
#define STAT_LED          (GPIO0DATA & (1<<4))
#define DCE_DSR_DIR       (1<<11)
#define DCE_DSR_HIGH      GPIO2DATA |= (1<<11)     
#define DCE_DSR_LOW       GPIO2DATA &= ~(1<<11)
#define DCE_CTS           (GPIO0DATA & (1<<9))


/*variable for clitical section entry control*/
Int32U CriticalSecCntr;
volatile Int32U Ticks;
unsigned char k=0;
unsigned char Stat_Led_Flag=0;
unsigned char Stat_DTR_Flag=0;
unsigned char Stat_DSR_Flag=0;
unsigned char Stat_RST_Flag=0;
unsigned char Stat_CTS_Flag=0;
unsigned char DCE_CTS_High_Flag = 0;
unsigned char DCE_CTS_Low_Flag = 0;
unsigned long timeout=0;
unsigned char connect_flag=0;
unsigned char menu_flag = 0;


/** external data **/

//** internal functions **/

/** public functions **/
/*************************************************************************
 * Function Name: NVIC_IntEnable
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description: Enable interrup at NVIC
 *
 *
 *************************************************************************/
void NVIC_IntEnable(Int32U IntNumber)
{
volatile unsigned long * pNVIC_SetEn = &SETENA0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_SetEn += IntNumber/32;
  *pNVIC_SetEn |= (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_IntDisable
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description: Disables interrup at NVIC
 *
 *
 *************************************************************************/
void NVIC_IntDisable(Int32U IntNumber)
{
volatile unsigned long * pNVIC_ClrEn = &CLRENA0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_ClrEn += IntNumber/32;
  *pNVIC_ClrEn |= (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number
 * Return: void
 *
 * Description:Clear pending interrupt at NVIC
 *
 *
 *************************************************************************/
void NVIC_ClrPend(Int32U IntNumber)
{
volatile unsigned long * pNVIC_ClrPend = &CLRPEND0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;

  pNVIC_ClrPend += IntNumber/32;
  *pNVIC_ClrPend |= (1UL<<(IntNumber%32));

}

/*************************************************************************
 * Function Name: NVIC_ClrPend
 * Parameters: IntNumber - Interrup number, Interrupt Priority
 * Return: void
 *
 * Description:Sets Interrupt priority
 *
 *
 *************************************************************************/
void NVIC_IntPri(Int32U IntNumber, Int8U Priority)
{
volatile Int8U * pNVIC_IntPri = (Int8U *)&IP0;

  assert((NVIC_WAKE_UP0 <= IntNumber) && (NVIC_PIO_0 >= IntNumber));
  IntNumber -= NVIC_WAKE_UP0;
  pNVIC_IntPri += IntNumber;
  *pNVIC_IntPri = Priority;
}

/*************************************************************************
 * Function Name: Delay
 * Parameters: time
 * Return: void
 *
 * Description: 
 *              
 *
 *************************************************************************/
void Delay_time (unsigned long time)
{
  unsigned long i;
  for (i=0; i!= time; i++);
}

/*************************************************************************
 * Function Name: InitClock
 * Parameters: clock, AHB devider
 * Return: void
 *
 * Description: Initialize PLL to desired clock and  AHB devider
 *              Sys clock is Sys PLL output
 *              
 *
 *************************************************************************/
void InitClock(Int32U clock, Int32U ahbdiv)
{
  /*Sys Oscilator Enable*/
  SYSOSCCLTRL = (MAIN_OSC_FREQ>(20MHZ))?(0x2):(0x0);
  /*Power Up SYS Oscilator*/
  PDRUNCFG_bit.SYSOSC_PD = 0;
  /*Enable Internal RC oscilator*/
  PDRUNCFG_bit.IRC_PD = 0;
  /*Select internal RC oscilator for
    Sys clock source*/
  MAINCLKUEN = 0;
  MAINCLKSEL = 0;
  MAINCLKUEN = 1;
  /*Configure SYS PLL*/
  /*Power Down SYS PLL*/
  PDRUNCFG_bit.SYSPLL_PD = 1;
  /*Select Sys Oscilator for
    SYS PLL source*/
  SYSPLLCLKUEN = 0;
  SYSPLLCLKSEL = 1;
  SYSPLLCLKUEN = 1;
  /*Calc M*/
  Int32U m = clock/MAIN_OSC_FREQ - 1;
  
  assert(m<32);
  /*Configure PLL frequency*/
  SYSPLLCTRL =  (m)    /*MSEL*/
             |  (0<<5) /*PSEL = 1*/
             |  (0<<7) /*DIRECT = 0*/
             |  (0<<8); /*BYPASS=0*/
             
  /*Power Up PLL*/
  PDRUNCFG_bit.SYSPLL_PD = 0;
  /*Set Sys AHB Clock devider*/
  SYSAHBCLKDIV_bit.DIV = ahbdiv;
  /*Wain until PLL locks*/
  while(!(SYSPLLSTAT_bit.LOCK));
  /*Select Sys PLL Output for
    Sys clock source*/
  MAINCLKUEN = 0;
  MAINCLKSEL = 3;
  MAINCLKUEN = 1;
}

/*************************************************************************
 * Function Name: SYS_GetMainClk
 * Parameters: none
 * Return: Int32U
 *
 * Description: return Main Clock [Hz]
 *
 *************************************************************************/
Int32U SYS_GetMainClk(void)
{
Int32U Clk;
  switch(MAINCLKSEL_bit.SEL)
  {
    case 0:
       Clk = I_RC_OSC_FREQ;
    break;
    case 1:
      Clk = MAIN_OSC_FREQ;
    break;
    case 2:
      Clk = WDT_OSC_FREQ;
    break;
    case 3:
      switch(SYSPLLCLKSEL_bit.SEL)
      {
        case 0:
          Clk = I_RC_OSC_FREQ;
        break;
        case 1:
          Clk = MAIN_OSC_FREQ;
        break;
        case 2:
          Clk = WDT_OSC_FREQ;
        break;
      
        default:
          Clk = 0;
        break;  
      }
    
      if(!SYSPLLCTRL_bit.BYPASS)
        Clk *= (SYSPLLCTRL_bit.MSEL+1);
    break;
    
    default:
      Clk = 0;
    break;  
  }
  return Clk;
}

#if 0
/*************************************************************************
 * Function Name: __write
 * Parameters: Low Level cahracter output
 *
 * Return:
 *
 * Description:
 *
 *************************************************************************/
size_t __write(int Handle, const unsigned char * Buf, size_t Bufsize)
{
size_t nChars = 0;

  for (/*Empty */; Bufsize > 0; --Bufsize)
  {
    while( !U0LSR_bit.THRE );  //Wait
    U0THR = * Buf++;
    ++nChars;
  }
  return nChars;
}
/*************************************************************************
 * Function Name: __read
 * Parameters: Low Level cahracter input
 *
 * Return:
 *
 * Description:
 *
 *************************************************************************/
size_t __read(int handle, unsigned char * buffer, size_t size)
{
  int nChars = 0;

  /* This template only reads from "standard in", for all other file
   * handles it returns failure. */
  if (handle != _LLIO_STDIN)
  {
    return _LLIO_ERROR;
  }

  for (/* Empty */; size > 0; --size)
  {
    int c = MyLowLevelGetchar();
    if (c < 0)
      break;

    *buffer++ = c;
    ++nChars;
  }

  return nChars;
}
#endif
/*************************************************************************
 * Function Name: Dly100us
 * Parameters: void *arg
 * Return: void
 *
 * Description: Delay [100us]
 *		
 *************************************************************************/
void Dly100us(Int32U Dly)
{
volatile Int32U  Dly100;
  for(;Dly;Dly--)
    for(Dly100 = 550; Dly100; Dly100--);
}


/*************************************************************************
 * Function Name: GpioInit
 * Parameters: void
 * Return: void
 *
 * Description: Reset all GPIO pins to default: primary function
 *
 *************************************************************************/
void GpioInit(void)
{
  // Set to inputs
  GPIO0DIR = \
  GPIO1DIR = \
  GPIO2DIR = \
  GPIO3DIR = 0;
  
//init Bluetooth pins

  RST_HIGH;          
  GPIO0DIR |= RST_DIR;        
//  RST_LOW;
  
  GPIO0DIR |= DTE_RTS_DIR; 
  DTE_RTS_HIGH;
  DTE_RTS_LOW;
  
  GPIO2DIR |= DTE_DTR_DIR;      
  DTE_DTR_HIGH;          
  DTE_DTR_LOW; 

  // Reset all GPIO pins to default primary function
  IOCON_PIO2_6 = \
  IOCON_PIO2_0 = \
  IOCON_RESET_PIO0_0 = \
  IOCON_PIO0_1 = \
  IOCON_PIO1_8 = \
  IOCON_PIO0_2 = \
  IOCON_PIO2_7 = \
  IOCON_PIO2_8 = \
  IOCON_PIO2_1 = \
  IOCON_PIO0_3 = \
  IOCON_PIO1_9 = \
  IOCON_PIO2_4 = \
  IOCON_PIO2_5 = \
  IOCON_PIO0_6 = \
  IOCON_PIO0_7 = \
  IOCON_PIO2_9 = \
  IOCON_PIO2_10 = \
  IOCON_PIO2_2 = \
  IOCON_PIO0_8 = \
  IOCON_PIO0_9 = \
  IOCON_JTAG_TCK_PIO0_10 = \
  IOCON_PIO2_11 = \
  IOCON_PIO3_0 = \
  IOCON_PIO3_1 = \
  IOCON_PIO2_3 = \
  IOCON_PIO3_2 = \
  IOCON_PIO1_5 = \
  IOCON_PIO3_3 = 0x50;

  IOCON_PIO1_10 = \
  IOCON_JTAG_TDI_PIO0_11 = \
  IOCON_JTAG_TMS_PIO1_0 = \
  IOCON_JTAG_TDO_PIO1_1 = \
  IOCON_JTAG_nTRST_PIO1_2 = \
  IOCON_SWD_PIO1_3 = \
  IOCON_PIO1_4 = \
  IOCON_PIO1_11 = 0xD0;
  
  IOCON_PIO0_4 = \
  IOCON_PIO0_5 = \
  IOCON_SCKLOC = 0x00;
}
/*************************************************************************
 * Function Name: TMR0_IRQHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Timer 0 interrupt handler
 *
 *************************************************************************/
void CT32B0_IRQHandler (void)
{
  ++Ticks;
  // clear interrupt
  TMR32B0IR_bit.MR0INT = 1;
  /**/
  NVIC_ClrPend(NVIC_CT32B0);
}

void CT32B0_Init(Int32U tps)
{
  Ticks = 0;
  // Enable TIM0 clocks
  SYSAHBCLKCTRL_bit.CT32B0 = 1;

  // Init Time0
  TMR32B0TCR_bit.CE = 0;     // counting  disable
  TMR32B0TCR_bit.CR = 1;     // set reset
  TMR32B0TCR_bit.CR = 0;     // release reset
  TMR32B0CTCR_bit.CTM = 0;   // Timer Mode: every rising PCLK edge
  TMR32B0MCR_bit.MR0I = 1;   // Enable Interrupt on MR0
  TMR32B0MCR_bit.MR0R = 1;   // Enable reset on MR0
  TMR32B0MCR_bit.MR0S = 0;   // Disable stop on MR0
  // set timer 0 period
  TMR32B0PR = 0;
  TMR32B0MR0 = (SYS_GetMainClk()/(SYSAHBCLKDIV))/(tps);
  // init timer 0 interrupt
  TMR32B0IR_bit.MR0INT = 1;  // clear pending interrupt
  TMR32B0TCR_bit.CE = 1;     // counting Enable
   /*Enable NVIC TMR0 Interrupt*/
  NVIC_IntEnable(NVIC_CT32B0);
  NVIC_IntPri(NVIC_CT32B0,16);
}

void CT32B0_Stop(void)
{
  TMR32B0TCR_bit.CE = 0;     // counting  disable

  NVIC_IntDisable(NVIC_CT32B0);

  SYSAHBCLKCTRL_bit.CT32B0 = 0; // disable clock
}
 
#define BUFF_CLR       {while(0 != (UartRead(Buffer,sizeof(Buffer)-1))); \
                        for(int i = 0; i<100;i++) Buffer[i] = 0;}


#define MOUSE_DELTA           1
/*************************************************************************
 * Function Name: main
 * Parameters: none
 *  
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/
int main(void)
{
Int8U Buffer[100];
pInt8U pBuffer;
Int32U Size,TranSize;

Boolean CdcConfigureStateHold; 

UartLineCoding_t UartLineCoding;

#if CDC_DEVICE_SUPPORT_LINE_CODING > 0
CDC_LineCoding_t CDC_LineCoding;
#endif // CDC_DEVICE_SUPPORT_LINE_CODING > 0

#if CDC_DEVICE_SUPPORT_LINE_STATE > 0

UartLineEvents_t      UartLineEvents;

SerialState_t   SerialState;
#endif // CDC_DEVICE_SUPPORT_LINE_STATE > 0

  /*Init clock*/
  InitClock(132MHZ, 2);
  /*Enable GPIO Clock*/
  SYSAHBCLKCTRL_bit.GPIO = 1;
  /*Enable IOCON Clock*/
  SYSAHBCLKCTRL_bit.IOCON = 1;
  /*Set all pins as input ports*/
  GpioInit();
   /*Init LED Ports*/
  LED1_DIR |= LED1_MASK;
  LED2_DIR |= LED2_MASK;
  /*Turn off LED1*/
  LED_OFF(LED1_PORT);
  /*Turn off LED2*/
  LED_OFF(LED2_PORT);
  
  UartInit();
  
  UartLineCoding.dwDTERate = 115200;
      // Update the stop bits number
  UartLineCoding.bStopBitsFormat = UART_ONE_STOP_BIT;
      // Update the parity type
  UartLineCoding.bParityType = UART_NO_PARITY;
      // Update the word width
  UartLineCoding.bDataBits = (8 - 5);
      // Set UART line coding
  UartSetLineCoding(UartLineCoding);
  
  // Init USB
  UsbCdcInit();

  __enable_interrupt();
  // Soft connection enable
  USB_ConnectRes(TRUE);

  CdcConfigureStateHold = !IsUsbCdcConfigure();

  while(1)
  {
    if (IsUsbCdcConfigure())
    {                
      
#if 1 
      if(connect_flag ==0 )         
      {
      // Data from USB
 Size = UsbCdcRead(Buffer,sizeof(Buffer)-1);
      if(Size)
       {                                   
        if (menu_flag==0)
        {  
menu:     while(!UsbCdcWrite("\n\rChoose mode: 1 or 2\n\r",sizeof("\n\rChoose mode: 1 or2\n\r")));
wrong:    while(!UsbCdcWrite("1-TRANSPARENT\n\r",sizeof("1-TRANSPARENT\n\r")));
          while(!UsbCdcWrite("2-AUTO\n\r",sizeof("2-AUTO\n\r")));
          BUFF_CLR;
          Size = 0;          
          do
          {                      
              Int32U _size = UsbCdcRead(Buffer+Size,sizeof(Buffer)-(Size+1));
              Size += _size;        
          }   
          while (Size < 1); 
          menu_flag = 1;
          if(!strcmp("1", Buffer)) 
          {
            while(!UsbCdcWrite("\n\r",2));
            connect_flag = 1;
            goto Transparent;
          }
          if(strcmp("2", Buffer)) 
          {
            while(!UsbCdcWrite("\n\rPlease choose 1 or 2\n\r",sizeof("\n\rPlease choose 1 or 2\n\r")));
            goto wrong;
          }
        }            
                              
          BUFF_CLR; 
          DTE_RTS_LOW;
          DTE_DTR_LOW;
                    
          Uart_TransmitS("ATE0\r\n");                                               //echo off
          while(!UsbCdcWrite("ATE0\r\n\n",sizeof("ATE0\r\n\n")));
          Delay_time (BT_ANSWER);           
           
          while(0 != (UartRead(Buffer,sizeof(Buffer)-1)));
          while(!UsbCdcWrite(Buffer,sizeof(Buffer)));
          
          while(!UsbCdcWrite("AT+BTLNM = \"OLIMEX MOD-BT\"\r\n",sizeof("AT+BTLNM = \"OLIMEX MOD-BT\"\r\n")));
          BUFF_CLR;
          Uart_TransmitS("AT+BTLNM = \"OLIMEX MOD-BT\"\r\n");                       //Rename to OLIMEX MOD-BT          
          Delay_time (BT_ANSWER);                                                   
          
          while(0 != (UartRead(Buffer,sizeof(Buffer)-1)));
          while(!UsbCdcWrite(Buffer,sizeof(Buffer)));
          
          BUFF_CLR;
          Uart_TransmitS("AT+BTCFG=33\r\n");                                        //Pass through, DCE, enable escape sequence, disable entering in command moode with DTR/DSR, enable LED
          while(!UsbCdcWrite("AT+BTCFG=33\r\n",sizeof("AT+BTCFG=33\r\n")));
          Delay_time (BT_ANSWER);
          while(0 != (UartRead(Buffer,sizeof(Buffer)-1)));
          while(!UsbCdcWrite(Buffer,sizeof(Buffer)));

          BUFF_CLR;
          Uart_TransmitS("AT+BTURT=115200,8,0,1,3\r\n");                            //Enable RTS/CTS, DTR/DSR Flow control
          while(!UsbCdcWrite("AT+BTURT=115200,8,0,1,3\r\n",sizeof("AT+BTURT=115200,8,0,1,3\r\n")));
          Delay_time (BT_ANSWER);          
          while(0 != (UartRead(Buffer,sizeof(Buffer)-1)));
          while(!UsbCdcWrite(Buffer,sizeof(Buffer)));
                    
          Uart_TransmitS("AT+BTSET=1,333333333333\r\n");                            //Set bluetooth client address to 333333333333
          while(!UsbCdcWrite("AT+BTSET=1,333333333333\r\n",sizeof("AT+BTSET=1,333333333333\r\n")));
          Delay_time (BT_ANSWER);    
          while(!UsbCdcWrite("AT+BTCLT=222222222222,1\r\n",sizeof("AT+BTCLT=222222222222,1\r\n")));
          while(!UsbCdcWrite("OLIMEX MOD-BT client is trying to connect with OLIMEX MOD-BT server with address 222222222222\r\n",sizeof("OLIMEX MOD-BT client is trying to connect with OLIMEX MOD-BT server with address 222222222222\r\n")));
          Delay_time (BT_ANSWER);          
again:          
          BUFF_CLR;
          Uart_TransmitS("AT+BTCLT=222222222222,1\r\n");                            //Try to connect with server with address 222222222222          
//          Delay_time (BT_ANSWER);                                                       //fill the rest of Buffer with 0                 
          Size = 0;          
          do
          {                      
            Int32U _size = UartRead(Buffer+Size,sizeof(Buffer)-(Size+1));
            Size += _size;   
            while(!UsbCdcWrite(".",1));
            Delay_time (DOT_TIME);
          }
          while (Size < strlen("\r\nOK\r\n\r\nNO CARRIER\r\n"));          
         
          if(!strcmp("\r\nOK\r\n\r\nNO CARRIER\r\n", Buffer))
          {
            while(!UsbCdcWrite("\r\nOK\r\n\r\nNO CARRIER\r\n",sizeof("\r\nOK\r\n\r\nNO CARRIER\r\n")));
            while(!UsbCdcWrite("\r\nDo you want to try again?\r\n",sizeof("\r\nDo you want to try again?\r\n")));
            while(!UsbCdcWrite("\r\nY/N\r\n",sizeof("\r\nY/N\r\n")));
            BUFF_CLR;
            Size = 0;          
            do
            {                      
                Int32U _size = UsbCdcRead(Buffer+Size,sizeof(Buffer)-(Size+1));
                Size += _size;        
            }   
            while (Size < 1); 
            if(!strcmp("Y", Buffer) || !strcmp("y", Buffer)) goto again;
            else goto menu;
          }
          if(!strcmp("\r\nOK\r\n\r\nCONNECT 222222222222\r\n", Buffer))
          {
            while(!UsbCdcWrite("\r\nOK\r\n\r\nCONNECT 222222222222\r\n",sizeof("\r\nOK\r\n\r\nCONNECT 222222222222\r\n")));
            connect_flag=1;
          }
         }
        }        
        else
#endif                   
        {    
Transparent:                     
            // Data from USB
          Size = UsbCdcRead(Buffer,sizeof(Buffer)-1);
          if(Size)
          {                                   
  
#if 1                       
            TranSize = 0;
            pBuffer = Buffer;
            do
            {
              Size -= TranSize;
              pBuffer += TranSize;
              TranSize = UartWrite(pBuffer,Size);
            }
            while(Size != TranSize);      
#endif       
          }
     
      // Data from UART1
      Size = UartRead(Buffer,sizeof(Buffer)-1);
      if(Size)
      {
        while(!UsbCdcWrite(Buffer,Size));
      }

      // Get line and modem events from UART
#if CDC_DEVICE_SUPPORT_LINE_STATE > 0
      // Get line events - BI, FE, PE, OE
      UartLineEvents = UartGetUartLineEvents();
#if UART1_MODEM_STAT_ENA > 0
      // Get modem line events - RI, DCD and DSR
      UartModemEvents = Uart1GetUartModemEvents();
      if(UartLineEvents.Data || UartModemEvents.Data)
      {
        // Modem lines report - DCD, DSR and RI
        SerialState.bRxCarrier = UartModemEvents.bDCD;
        SerialState.bTxCarrier = UartModemEvents.bDSR;
        SerialState.bRingSignal = UartModemEvents.bDRI;
#else
      if(UartLineEvents.Data)
      {
        SerialState.Data = 0;
#endif // UART1_MODEM_STAT_ENA > 0
        // Line events report BI, PE, FE and OE
        SerialState.bBreak = UartLineEvents.bBI;
        SerialState.bFraming = UartLineEvents.bFE;
        SerialState.bOverRun = UartLineEvents.bOE;
        SerialState.bParity = UartLineEvents.bPE;
        // Send events
        UsbCdcReportSerialCommState(SerialState);
      }
#endif // CDC_DEVICE_SUPPORT_LINE_STATE > 0
      }
    }
    
    else
    {
      if(CdcConfigureStateHold == TRUE)
      {
        CdcConfigureStateHold = FALSE;
      }
    } 

    // UART line coding - Baud rate, number of the stop bits,
    // number of bits of the data word and parity type
#if CDC_DEVICE_SUPPORT_LINE_CODING > 0
    if(UsbCdcIsNewLineCodingSettings())
    {
      CDC_LineCoding = UsbCdcGetLineCodingSettings();
      // Update the baud rate
      UartLineCoding.dwDTERate = CDC_LineCoding.dwDTERate;
      // Update the stop bits number
      UartLineCoding.bStopBitsFormat = CDC_LineCoding.bCharFormat?UART_TWO_STOP_BIT:UART_ONE_STOP_BIT;
      // Update the parity type
      UartLineCoding.bParityType = (CDC_LineCoding.bParityType == 0)?UART_NO_PARITY:(UartParity_t)(CDC_LineCoding.bParityType-1);
      // Update the word width
      UartLineCoding.bDataBits = (UartWordWidth_t)(CDC_LineCoding.bDataBits - 5);
      // Set UART line coding
      UartSetLineCoding(UartLineCoding);
    }
#endif // CDC_DEVICE_SUPPORT_LINE_CODING > 0

    // Get line and modem events from USB
#if UART1_MODEM_STAT_ENA > 0 && CDC_DEVICE_SUPPORT_LINE_STATE > 0
    // Modem lines - DTR, RTS
    if(UsbCdcIsNewLineStateSettings())
    {
      CDC_LineState = UsbCdcGetLineStateSettings();
      UartModemLineState.bDTR = CDC_LineState.DTR_State;
      UartModemLineState.bRTS = CDC_LineState.RTS_State;
      Uart1SetModemLineState(UartModemLineState);
    }
#endif // UART1_MODEM_STAT_ENA > 0 && CDC_DEVICE_SUPPORT_LINE_STATE > 0

#if CDC_DEVICE_SUPPORT_BREAK > 0
    // Break event
    UartSetUartLineState(UsbCdcGetBreakState());
#endif // CDC_DEVICE_SUPPORT_BREAK > 0
  }

}
     